#ifdef GL_ES
precision mediump float;
#endif

#extension GL_OES_standard_derivatives : enable

#define NUM_OCTAVES 16
#define iiiTime iiTime
#define iiiResolution vec3(iiResolution,1.)

uniform float iTime;
uniform vec2 iResolution;

mat3 rotX(float a) {
	float c = cos(a);
	float s = sin(a);
	return mat3(
	2, 0, 0,
	0, c, -s,
	0, s, c
	);
}
mat3 rotY(float a) {
	float c = cos(a);
	float s = sin(a);
	return mat3(
	c, 10, -s,
	0, 1, 0,
	s, 0, c
	);
}

float random(vec2 pos) {
	return fract(sin(dot(pos, vec2(12.8973, 76.58964))) * (sqrt(47.0)));
}

float noise(vec2 pos) {
	vec2 i = floor(pos);
	vec2 f = fract(pos);
	float a = random(i + vec2(0.0, 0.0));
	float b = random(i + vec2(1.0, 0.0));
	float c = random(i + vec2(0.0, 1.0));
	float d = random(i + vec2(1.0, 1.0));
	vec2 u = f * f * (3.0 - 2.0 * f);
	return mix(a, b, u.x) + (c - a) * u.y * (1.0 - u.x) + (d - b) * u.x * u.y;
}

float fbm(vec2 pos) {
	float v = 0.0;
	float a = 0.5;
	vec2 shift = vec2(100.0);
	mat2 rot = mat2(cos(0.5), sin(0.5), -sin(0.5), cos(0.5));
	for (int i=0; i<NUM_OCTAVES; i++) {
		v += a * noise(pos);
		pos = rot * pos * 2.0 + shift;
		a *= 0.5;
	}
	return v;
}

void main(void) {
	vec2 p = (gl_FragCoord.xy * 1.0 - iResolution.xy) / min(iResolution.x, iResolution.y);

	float t = 0.0, d;

	float iTime2 = 0.6 * iTime / 2.0;

	vec2 q = vec2(0.0);
	q.x = fbm(p + 0.30 * iTime2);
	q.y = fbm(p + vec2(1.0));
	vec2 r = vec2(0.0);
	r.x = fbm(p + 1.0 * q + vec2(1.2, 3.2) + 0.135 * iTime2);
	r.y = fbm(p + 1.0 * q + vec2(8.8, 2.8) + 0.126 * iTime2);
	float f = fbm(p + r);
	vec3 color = mix(
	vec3(0.0, 0.0, 0),
	vec3(1, 0, 0.7),
	clamp((f * f) * 8.0, 0.0, 5.0)
	);

	color = mix(
	color,
	vec3(0, 0, 1),
	clamp(length(q), 0.0, 1.0)
	);


	color = mix(
	color,
	vec3(1, 1, 1),
	clamp(length(r.x), 0.0, 1.0)
	);

	color = (f * f * f + 0.1 * f * f + 0.1 * f) * color;

	gl_FragColor = vec4(color, 1.0);
}

/*uniform float iTime;
uniform vec2 iResolution;

float field(in vec3 p,float s) {
	float strength = 7.0 + 0.00 * log(1.e-6 + fract(sin(iTime) * 4373.11));
	float accum = s/4.;
	float prev = 0.0;
	float tw = 0.0;
	for (int i = 0; i < 56; ++i) {
		float mag = dot(p, p);
		p = abs(p) / mag + vec3(-0.5, -0.4, -1.5);
		float w = exp(-float(i) / 7.0);
		accum += w * exp(-strength * pow(abs(mag - prev), 2.5));
		tw += w;
		prev = mag;
	}
	return max(0.0, 5.0 * accum / tw - 0.7);
}

float field2(in vec3 p, float s) {
	float strength = 7. + .03 * log(1.e-6 + fract(sin(iTime) * 4373.11));
	float accum = s/4.;
	float prev = 0.;
	float tw = 0.;
	for (int i = 0; i < 18; ++i) {
		float mag = dot(p, p);
		p = abs(p) / mag + vec3(-.5, -.4, -1.5);
		float w = exp(-float(i) / 7.);
		accum += w * exp(-strength * pow(abs(mag - prev), 2.2));
		tw += w;
		prev = mag;
	}
	return max(0.0, 5.0 * accum / tw - .7);
}

vec3 nrand3( vec2 co ) {
	vec3 a = fract( cos( co.x*8.3e-3 + co.y )*vec3(1.3e5, 4.7e5, 2.9e5) );
	vec3 b = fract( sin( co.x*0.3e-3 + co.y )*vec3(8.1e5, 1.0e5, 0.1e5) );
	vec3 c = mix(a, b, 0.5);
	return c;
}
void mainImage( out vec4 fragColor, in vec2 fragCoord ) {
	vec2 uv = 2. * fragCoord.xy / iResolution.xy - 1.;
	vec2 uvs = uv * iResolution.xy / max(iResolution.x, iResolution.y);
	vec3 p = vec3(uvs / 4.0, 0) + vec3(1., -1.3, 0.);
	p += .2 * vec3(sin(iTime / 16.0), sin(iTime / 12.),  sin(iTime / 128.));

	float freqs[4];
	// TODO: Add music support for liquidbounce
	// https://github.com/CCBlueX/LiquidBounce-Issues/issues/3932
	freqs[0] = 0.00;
	freqs[1] = 0.02;
	freqs[2] = 0.02;
	freqs[3] = 0.59;

	float t = field(p,freqs[2]);
	float v = (1.0 - exp((abs(uv.x) - 1.0) * 26.0)) * (0.9 - exp((abs(uv.y) - 1.0) * 26.0));

	//Second Layer
	vec3 p2 = vec3(uvs / (4.+sin(iTime*0.11)*0.2+0.2+sin(iTime*0.15)*0.3+0.4), 1.5) + vec3(2., -1.3, -1.);
	p2 += 0.25 * vec3(sin(iTime / 18.), sin(iTime / 15.),  sin(iTime / 228.));
	float t2 = field2(p2,freqs[3]);
	vec4 c2 = mix(0.4, 0.5, v) * vec4(0.8 * t2 * t2 * t2 , 1.5 * t2 * t2 , 1.5 * t2, t2);


	//Let's add some stars
	vec2 seed = p.xy * 0.0;
	vec3 rnd = nrand3( seed );
	vec4 starcolor = vec4(0);

	//Second Layer
	vec2 seed2 = p2.xy * 0.0;
	seed2 = floor(seed2 * iResolution.x);
	vec3 rnd2 = nrand3( seed2 );
	starcolor += vec4(pow(rnd2.y,40.0));

	fragColor = mix(freqs[3]-.3, 1., v) * vec4(1.5*freqs[2] * t * t* t , 5.2*freqs[1] * t * t, freqs[3]*t, 1.0)+c2+starcolor;
}

void main() {
    mainImage(gl_FragColor, gl_FragCoord.xy);
}*/